# DISCUS Diffuse Scattering and defect structure simulation   V3.0
# ================================================================
#
# This is the Makefile for DISCUS on platforms using f77. 
# Uncomment the compiler options etc. for your platform from
# the list below. If you platform is not listed and you succeed
# installing the program, please send us a short notice.
#
# (c) Th. Proffen    (proffen@rsc.anu.edu.au)
#     R.B. Neder     (neder@yoda.kri.physik.uni-muenchen.de)
#

# Where to install the binary and help file and a backup location

BINDIR  = $(HOME)/bin
BACKDIR = $(HOME)/develop/discus-3.0/discus

# Your compiler name and install program

F77     = f77
CC      = cc
INSTALL = install -c
#INSTALL = install.sh -c

# Here are the platform specific definitions
# If you want line editing add -DREADLINE on
# the CFLAGS line. If you have trouble with interactiv
# input, remove -DREADLINE.

##########################
# SUN Sparc Workstation
##########################
CFLAGS  = -DREADLINE
FFLAGS  = -Nl80 -C -g
LDFLAGS = -lm -lV77
SYSOBJ  = appl_sun.o sys_sun.o
SYSSRC  = appl_sun.f sys_sun.f

##########################
# HP/UX Workstation
##########################
#CFLAGS  = -DREADLINE
#FFLAGS  = -C -O +Obb2048 +E1
#LDFLAGS = 
#SYSOBJ  = appl_hpu.o sys_hpu.o
#SYSSRC  = appl_hpu.f sys_hpu.f

##########################
# DEC Alpha
##########################
#CFLAGS  = -DREADLINE
#FFLAGS  = -O
#LDFLAGS = 
#SYSOBJ  = appl_sun.o sys_sun.o
#SYSSRC  = appl_sun.f sys_sun.f

##########################
# SGI Power Challange
##########################
#CFLAGS  = 
#FFLAGS  = -C  -O -Olimit 3000
#LDFLAGS = 
#SYSOBJ  = appl_hpu.o sys_hpu.o
#SYSSRC  = appl_hpu.f sys_hpu.f

##########################
# SGI Workstations IRIS
##########################
#CFLAGS  = 
#FFLAGS  = -O3
#LDFLAGS = 
#SYSOBJ  = appl_sun.o sys_sun.o
#SYSSRC  = appl_sun.f sys_sun.f

##########################
# VPP300 Computer
##########################
#CFLAGS  = -DREADLINE
#FFLAGS  = -Wv,-qv -Ob -X9
#LDFLAGS = 
#SYSOBJ  = appl_hpu.o sys_vpp.o
#SYSSRC  = appl_hpu.f sys_vpp.f
#F77     = frt

##########################
# There should be no need to change anything below this line

OBJ   =	discus.o addfile.o blk_appl.o blockdat.o dkdo.o \
        err_appl.o errlist.o exit.o fourier.o graphic.o help.o kdo.o \
        lazy.o learn.o macro.o metric.o interpret.o micro.o modify.o \
        patters.o plot.o save.o show.o stack.o structur.o sup.o symm.o \
        tensors.o  thermal.o trafo.o upd_par.o waves.o chem.o \
        rmc.o readline.o extrmc.o $(SYSOBJ)

FSRC  =	discus.f addfile.f blk_appl.f blockdat.f dkdo.f \
       	err_appl.f errlist.f exit.f fourier.f graphic.f help.f kdo.f \
	lazy.f learn.f macro.f metric.f interpret.f micro.f modify.f \
	patters.f plot.f save.f show.f stack.f structur.f sup.f symm.f \
        tensors.f  thermal.f trafo.f upd_par.f waves.f chem.f \
	rmc.f extrmc.f $(SYSSRC)

CSRC  = readline.c

discus: $(OBJ)
	$(F77) $(FFLAGS) $(LDFLAGS) -o discus $(OBJ)

clean: 
	rm -f discus $(OBJ) 

install: discus discus.hlp color.map
	$(INSTALL) discus $(BINDIR)
	$(INSTALL) discus.hlp $(BINDIR)
	$(INSTALL) color.map $(BINDIR)

backup: *.f sys_*cc.c read*.c *.inc *.cmm *.hlp *.map *ake* c*.lah CHANGES.LOG
	tar -cvf discus.tar *.f read*.c sys_*cc.c *.inc *.cmm *.hlp  \
	                       *.map *ake* c*.lah CHANGES.LOG
	gzip discus.tar
	mv discus.tar.gz $(BACKDIR)

# Here are the dependencies created with 'mkmf'
# Do not edit or delete
############################################################################
############################################################################
###
addfile.o: config.inc diffuse.inc errlist.inc

appl_f2c.o: envir.inc prompt.inc

appl_hpu.o: envir.inc prompt.inc

appl_lah.o: envir.inc errlist.inc prompt.inc

appl_sun.o: envir.inc prompt.inc

appl_vax.o: envir.inc errlist.inc prompt.inc

blk_appl.o: atom_env.inc chem.inc config.inc crystal.cmm crystal.inc \
         diffuse.inc envir.inc errlist.inc microdis.inc microsta.inc \
         output.inc param.inc patters.inc plot.inc rmc.inc stack.inc \
         symm.inc waves.inc

blockdat.o: debug.inc doloop.inc learn.inc macro.inc param.inc

chem.o: atom_env.inc chem.inc config.inc crystal.cmm crystal.inc \
         doact.inc errlist.inc learn.inc macro.inc param.inc prompt.inc

discus.o: doact.inc errlist.inc learn.inc macro.inc prompt.inc wink.inc

dkdo.o: charact.inc doact.inc doexec.inc doloop.inc errlist.inc learn.inc \
         macro.inc prompt.inc random.inc wink.inc

err_appl.o: errlist.inc

errlist.o: errlist.inc prompt.inc

extrmc.o: chem.inc config.inc crystal.cmm crystal.inc errlist.inc \
         random.inc rmc.inc

fourier.o: config.inc crystal.cmm crystal.inc diffuse.inc doact.inc \
         errlist.inc learn.inc macro.inc output.inc prompt.inc random.inc

graphic.o: config.inc diffuse.inc doact.inc envir.inc errlist.inc \
         learn.inc macro.inc output.inc patters.inc prompt.inc random.inc

help.o: doact.inc envir.inc errlist.inc macro.inc prompt.inc

kdo.o: errlist.inc inverse.inc learn.inc

learn.o: errlist.inc learn.inc prompt.inc

macro.o: charact.inc doact.inc errlist.inc macro.inc prompt.inc

metric.o: config.inc crystal.cmm crystal.inc errlist.inc param.inc \
         wink.inc

micro.o: config.inc crystal.cmm crystal.inc doact.inc errlist.inc \
         learn.inc macro.inc micro.inc microdis.inc prompt.inc random.inc \
         wink.inc

modify.o: atom_env.inc charact.inc config.inc crystal.cmm crystal.inc \
         errlist.inc param.inc

patters.o: config.inc crystal.cmm crystal.inc diffuse.inc doact.inc \
         errlist.inc intens.inc inverse.inc learn.inc macro.inc output.inc \
         patters.inc prompt.inc recipro.inc

plot.o: config.inc crystal.cmm crystal.inc doact.inc errlist.inc \
         learn.inc macro.inc microdis.inc microsta.inc plot.inc prompt.inc

rmc.o: atom_env.inc chem.inc config.inc crystal.cmm crystal.inc \
         diffuse.inc doact.inc errlist.inc generate.inc learn.inc macro.inc \
         param.inc prompt.inc random.inc rmc.inc

save.o: config.inc crystal.cmm crystal.inc errlist.inc

show.o: config.inc crystal.cmm crystal.inc envir.inc errlist.inc \
         macro.inc param.inc

stack.o: config.inc crystal.cmm crystal.inc diffuse.inc doact.inc \
         errlist.inc learn.inc macro.inc microsta.inc prompt.inc random.inc \
         stack.inc symm.inc

structur.o: config.inc crystal.cmm crystal.inc doact.inc errlist.inc \
         generate.inc learn.inc macro.inc prompt.inc recipro.inc

sup.o: doact.inc errlist.inc learn.inc macro.inc param.inc prompt.inc \
         random.inc

symm.o: config.inc crystal.cmm crystal.inc doact.inc errlist.inc \
         learn.inc macro.inc microdis.inc microsta.inc prompt.inc symm.inc

sys_f2c.o: envir.inc errlist.inc wink.inc

sys_f90.o: wink.inc

sys_hpu.o: envir.inc errlist.inc

sys_lah.o: envir.inc errlist.inc

sys_sun.o: envir.inc errlist.inc

sys_vax.o: envir.inc errlist.inc

sys_vpp.o: envir.inc errlist.inc wink.inc

tensors.o: errlist.inc wink.inc

thermal.o: config.inc crystal.cmm crystal.inc errlist.inc random.inc

upd_par.o: atom_env.inc config.inc crystal.cmm crystal.inc errlist.inc \
         microdis.inc param.inc

waves.o: charact.inc config.inc crystal.cmm crystal.inc doact.inc \
         errlist.inc learn.inc macro.inc microdis.inc microsta.inc \
         prompt.inc random.inc waves.inc wink.inc
