c*****7*****************************************************************
	subroutine waves
c-
c	calculates the displacement of the atoms due to a plane wave 
c	travelling through the crystal
c
c	Version  : 1.1
c	Date     : 11 Sep 92
c
c	Author   : R.B. Neder  (neder@yoda.kri.physik.uni-muenchen.de)
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'doact.inc'
	include       'errlist.inc'
	include       'microsta.inc'
	include       'learn.inc'
	include       'macro.inc'
	include       'prompt.inc'
	include       'waves.inc'
c
	integer        long,itrans,dens,maxw
	PARAMETER     (LONG=1,ITRANS=2,DENS=3,maxw=MAXSCAT)
	logical        lnew,lold
	parameter     (lnew=.true.,lold=.false.)
c
	character*5    befehl
	character*6    cphase(2)
	character*11   prom
	character*16   cwave(3)
	character*28   clong(2,2)
	character*200  line,zeile,cpara(maxw)
	integer        lpara(maxw),lp,length,lbef
	integer        indxg,ianz,i,iwv_func
	logical        lend
	real           werte(maxw)
c
	logical        str_comp
c
	data cphase    /'random','fixed '/
	data clong     /' longitudinal accoustic wave',
     &	                ' transversal  accoustic wave',
     &	                ' longitudinal optical   wave',
     &	                ' transversal  optical   wave'/
	data cwave     /' box shaped wave',
     &	                ' sinusoidal wave',
     &	                ' triangular wave'/
	data iwv_func  /2/
c
	lend=.false.
	call no_error
c
	do while (.not. lend)
	  prom=prompt//'/wave'
	  call get_command (line,length,befehl,lbef,zeile,lp,prom)
	  if(line.ne.' ' .and. line(1:1).ne.'#') then
c
c------	----search for "="
c
	    indxg=index(line,'=')
	    if(indxg.ne.0 
     &	              .and. .not.(str_comp(befehl,'echo',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'syst',2,lbef,4))
     &	              .and. .not.(str_comp(befehl,'help',2,lbef,4) .or.
     &	                          str_comp(befehl,'?   ',2,lbef,4)) )then
c
c------	------evaluatean expression and assign the value to a variabble
c
	      call do_math(line,indxg,length)
	    else
c
c------ ----execute a macro file
c
            if(befehl(1:1).eq.'@') then
	      if(length.ge.2) then
	        call file_kdo(line(2:length),length-1)
	      else
	        ier_num = -13
	        ier_typ = ER_MAC
	      endif
c
c------	----accoustic wave 'acco'
c
	    elseif(str_comp(befehl,'acco',2,lbef,4)) then
	      lacoust=.true.
c
c------	----amplitude of wave 'ampl'
c
	    elseif(str_comp(befehl,'ampl',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          amp=werte(1)
	        endif
	      endif
c
c------	----list asymmetric unit 'asym'
c
	    elseif(str_comp(befehl,'asym',2,lbef,4)) then
	      call show_asym
c
c------	----list atoms present in the crystal 'chem'
c
	    elseif(str_comp(befehl,'chem',2,lbef,4)) then
	      call show_chem
c
c------	----continues a macro 'continue'
c
	    elseif(str_comp(befehl,'continue',2,lbef,8)) then
	      call macro_continue(zeile,lp)
c
c------	----density wave 'dens'
c
	    elseif(str_comp(befehl,'dens',3,lbef,4)) then
	      iwave=DENS
c
c------	----Deselect which atoms are included in the wave 'dese'
c
	    elseif(str_comp(befehl,'dese',3,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 ) then
	        if(cpara(1)(1:3).eq.'mic') then
	          wv_md_s  = MD_S_EVE
	          wv_md_nr = 0
	        else
	          call get_iscat(ianz,cpara,lpara,werte,maxw,lold)
	          if(ier_num.eq.0) then
	            if(werte(1).eq.-1) then
	              do i=1,cr_nscat
	                w_latom(i) = .false.
	                w_repl(i)  = 0
	              enddo
	            else
	              do i=1,ianz
	                w_latom(nint(werte(i))) = .false.
	                w_repl (nint(werte(i))) = 0
	              enddo
	            endif
	          endif
	        endif
	      endif
c
c------ ----Echo a string, just for interactive check in a macro 'echo'
c
	    elseif(str_comp(befehl,'echo',2,lbef,4)) then
              write(*,'(1x,a)') zeile(1:79)
c
c------	 ---Evaluate an expression, just for interactive check 'eval'
c
	    elseif(str_comp(befehl,'eval',2,lbef,4)) then
	      call do_eval(zeile,lp)
c
c------	----exit 'exit'
c
	    elseif(str_comp(befehl,'exit',2,lbef,4)) then
	      lend=.true.
c
c------	----wave function 'func'
c	    possible functions are 'box', 'sinusoidal', 'triangular'
c
	    elseif(str_comp(befehl,'func',1,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        wv_func = cpara(1)
	        if(wv_func(1:1).eq.'b') then
	          iwv_func = 1
	        elseif(wv_func(1:1).eq.'s') then
	          iwv_func = 2
	        elseif(wv_func(1:1).eq.'t') then
	          iwv_func = 3
	        endif
	      endif
c
c------	----help 'help','?'
c
	    elseif(str_comp(befehl,'help',2,lbef,4) .or.
     &	           str_comp(befehl,'?   ',1,lbef,4)     ) then
	      if(str_comp(zeile,'errors',2,lp,6) ) then
	        lp = lp + 7
	        call do_hel('discus '//zeile,lp)
	      else
	        lp = lp + 12
	        call do_hel('discus wave '//zeile,lp)
	      endif
c
c------	----wave length 'leng'
c
	    elseif(str_comp(befehl,'leng',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          rlam=werte(1)
	        endif
	      endif
c
c------	----longitudinal wave 'long'
c
	    elseif(str_comp(befehl,'long',2,lbef,4)) then
	      iwave=LONG
c
c------	----optical   wave 'opti'
c
	    elseif(str_comp(befehl,'opti',2,lbef,4)) then
	      lacoust=.false.
c
c------	----direction of oscillation 'osci'
c
	    elseif(str_comp(befehl,'osci',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.3) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          swing(1)=werte(1)
	          swing(2)=werte(2)
	          swing(3)=werte(3)
	        endif
	      endif
c
c------	----initial phase 'phas'
c
	    elseif(str_comp(befehl,'phas',3,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        if(str_comp(cpara(1),'rand',1,lpara(1),4)) then
	          wv_phase_typ = WV_RAND
	        else
	          wv_phase_typ = WV_FIX
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            phase=werte(1)
	          endif
	        endif
	      endif
c
c------	----upper limit of probability for density waves 'phig'
c
	    elseif(str_comp(befehl,'phig',3,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          phigh = min(1.0,abs(werte(1)))
	        endif
	      endif
c
c------	----lower limit of probability for density waves 'plow'
c
	    elseif(str_comp(befehl,'plow',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.1) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          plow = min(1.0,abs(werte(1)))
	        endif
	      endif
c
c------	----Select which atoms are replaced by the wave 'repl'
c
	    elseif(str_comp(befehl,'repl',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(cpara(1)(1:3).eq.'mic' .and. ianz.eq.2) then
	          if(cpara(2)(1:3).eq.'all') then
	            wv_md_s  = MD_S_ALL
	            wv_md_nr = 0
	          elseif(cpara(2)(1:3).eq.'non') then
	            wv_md_s  = MD_S_NON
	            wv_md_nr = 0
	          elseif(cpara(2)(1:3).eq.'eve') then
	            wv_md_s  = MD_S_EVE
	            wv_md_nr = 0
	          else
	            cpara(1) = '0.0'
	            lpara(1) = 3
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            wv_md_nr = nint(werte(2))
	            wv_md_s  = MD_S_IND
	          endif
	        elseif(ianz.eq.1 .or. ianz.eq.2 ) then
	          call get_iscat(ianz,cpara,lpara,werte,maxw,lnew)
	          if(ier_num.eq.0) then
	            if(werte(1).eq.-1) then
	              do i=1,cr_nscat
	                w_latom(i) = .true.
	                w_repl(i)  = nint(werte(2))
	              enddo
	            else
	              w_latom(nint(werte(1))) = .true.
	              w_repl (nint(werte(1))) = nint(werte(2))
	            endif
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----run waves 'run'
c
	    elseif(str_comp(befehl,'run ',2,lbef,4)) then
	      call do_wave
c
c------	----Select which atoms are included in the wave 'sele'
c
	    elseif(str_comp(befehl,'sele',2,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(cpara(1)(1:3).eq.'mic' .and. ianz.eq.2) then
	          if(cpara(2)(1:3).eq.'all') then
	            wv_md_s  = MD_S_ALL
	            wv_md_nr = 0
	          elseif(cpara(2)(1:3).eq.'non') then
	            wv_md_s  = MD_S_NON
	            wv_md_nr = 0
	          elseif(cpara(2)(1:3).eq.'eve') then
	            wv_md_s  = MD_S_EVE
	            wv_md_nr = 0
	          else
	            cpara(1) = '0.0'
	            lpara(1) = 3
	            call ber_params(ianz,cpara,lpara,werte,maxw)
	            wv_md_nr = nint(werte(2))
	            wv_md_s  = MD_S_IND
	          endif
	        else
	          call get_iscat(ianz,cpara,lpara,werte,maxw,lold)
	          if(ier_num.eq.0) then
	            if(werte(1).eq.-1) then
	              do i=1,cr_nscat
	                w_latom(i) = .true.
	                w_repl(i)  = 0
	              enddo
	            else
	              do i=1,ianz
	                w_latom(nint(werte(i))) = .true.
	                w_repl (nint(werte(i))) = 0
	              enddo
	            endif
	          endif
	        endif
	      endif
c
c------	----set constant shift 'shif'
c
	    elseif(str_comp(befehl,'shif',3,lbef,4)) then
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0) then
	        if(ianz.eq.1) then
	          call ber_params(ianz,cpara,lpara,werte,maxw)
	          if(ier_num.eq.0) then
	            amp0=werte(1)
	          endif
	        else
	          ier_num = -6
	          ier_typ = ER_COMM
	        endif
	      endif
c
c------	----show current parameters 'show'
c
	    elseif(str_comp(befehl,'show',3,lbef,4)) then
	      if(iwave.eq.DENS) then
	        write(*,3005)
	        write(*,3006) cwave(iwv_func)
	        write(*,3010) wave
	        write(*,3035) rlam,plow,phigh,phase,
     &	                      cphase(wv_phase_typ)
	        write(*,3036) cmic_s(wv_md_s)
	        if(wv_md_s.eq.MD_S_IND) then
	          write(*,3037) wv_md_nr
	        endif
	        write(*,3040)
	        write(*,3042)
	        do i=1,cr_nscat
	          if(w_latom(i)) then
	            write(*,3055) i,cr_at_lis(i),
     &	                          w_repl(i),cr_at_lis(w_repl(i))
	          endif
	        enddo
	      else
	        if(lacoust) then
	          write(*,3000) clong(iwave,1)
	        else
	          write(*,3000) clong(iwave,2)
	        endif
	        write(*,3006) cwave(iwv_func)
	        write(*,3010) wave
	        if(iwave.eq.ITRANS) write(*,3020) swing
	        write(*,3030) rlam,amp,amp0,phase,
     &	                      cphase(wv_phase_typ)
	        write(*,3036) cmic_s(wv_md_s)
	        if(wv_md_s.eq.MD_S_IND) then
	          write(*,3037) wv_md_nr
	        endif
	        write(*,3040)
	        write(*,3041)
	        do i=1,cr_nscat
	          if(w_latom(i)) then
	            write(*,3050) i,cr_at_lis(i)
	          endif
	        enddo
	      endif
c
c------	----transversal wave 'tran'
c
	    elseif(str_comp(befehl,'tran',1,lbef,4)) then
	      iwave=ITRANS
c
c------	----direction of wave vector 'vect'
c
	    elseif(str_comp(befehl,'vect',1,lbef,4)) then
	      ier_num = -6
	      ier_typ = ER_COMM
	      call  get_params(zeile,ianz,cpara,lpara,maxw,lp)
	      if(ier_num.eq.0 .and. ianz.eq.3) then
	        call ber_params(ianz,cpara,lpara,werte,maxw)
	        if(ier_num.eq.0) then
	          wave(1)=werte(1)
	          wave(2)=werte(2)
	          wave(3)=werte(3)
	        endif
	      endif
c
c------- -Operating System Kommandos 'syst'
c
	  elseif(str_comp(befehl,'syst',2,lbef,4)) then
	    if(zeile.ne. ' ') then
	      call do_operating(zeile(1:lp))
	    else
	      ier_num = -6
	      ier_typ = ER_COMM
	    endif
c
c
c------  -----waiting for user input
c
	    elseif(str_comp(befehl,'wait',3,lbef,4)) then
	      call do_input(zeile,lp)
c
	    else
	      ier_num = -8
	      ier_typ = ER_COMM
	    endif
	    endif
	  endif
	  if(ier_num.ne.0) then
	    call errlist
	    if(lmakro) then
	      call macro_close
	      prompt_status = PROMPT_ON
	    endif
	    if(lblock) then
	      ier_num = -11
	      ier_typ = ER_COMM
	      return
	    endif
	    call no_error
	  endif
	enddo
9999	continue
c
2001	format(1x,'macro ',a)
3000	format(' Wave type         : ',a)
3005	format(' Wave type         : Density wave')
3006	format(' wave function     : ',a16)
3010	format(' wave vector       : ',3(2x,f9.4))
3020	format(' oscillation vector: ',3(2x,f9.4))
3030	format(' wave length       : ',2x,f9.4/
     &	       ' amplitude         : ',2x,f9.4/
     &	       ' constant shift    : ',2x,f9.4/
     &	       ' phase at 0,0,0    : ',2x,f9.4/
     &	       ' phase status      : ',2x,a   )
3035	format(' wave length       : ',2x,f9.4/
     &	       ' low  probabilty   : ',2x,f9.4/
     &	       ' high probabilty   : ',2x,f9.4/
     &	       ' phase at 0,0,0    : ',2x,f9.4/
     &	       ' phase status      : ',2x,a   )
3036	format(' select status     : ',a)
3037	format(22x,i4)
3040	format(' selected atoms    :')
3041	format('                      type name')
3042	format('                      selected   replaced by'/
     &	       '                      type name  type name')
3045	format(' none               ')
3050	format(22x,2x,i2,1x,a4)
3055	format(20x,2(4x,i2,1x,a4))
      END
c*****7*********************************************************
	subroutine do_wave
c-
c	Main waves routine	
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microsta.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	integer        long,itrans,dens
	PARAMETER     (LONG=1,ITRANS=2,DENS=3)
c
	integer        i,lbef
	real           wavec(3),swingc(3),disp(3),disc(3)
	real           wavep(3),swingp(3),ww,rnn
	real           quad
c
	integer        len_str
	logical        str_comp
c
	real           sinus,box,triang
	external       sinus,box,triang
c
c------	normalise wavevector to length 1 A,
c------	transform to normalised space
c
	ww=sqrt(quad(wave,wave,cr_gten))
	do i=1,3
	 wavec(i)=wave(i)/ww
	enddo 
	call trans(wavec,cr_fmat,wavep,3)
c
c------	For density waves set correct values of amp and amp0 depending
c------	on wave function type
c
	if(iwave.eq.DENS) then
	  lbef = len_str(wv_func)
	  if(str_comp(wv_func,'box',1,lbef,3)) then
	    amp  =  phigh-plow
	    amp0 =  plow
	  elseif(str_comp(wv_func,'sinu',1,lbef,4)) then
	    amp  = (phigh-plow)*0.5
	    amp0 = (phigh+plow)*0.5
	  elseif(str_comp(wv_func,'tria',1,lbef,4)) then
	    amp  =  phigh-plow
	    amp0 =  plow
	  endif
	else
c
c------	normalise oscillation vector to lenght 1 A,
c------	transform to normalised space
c
	  if(iwave.eq.itrans) then
	    rnn=sqrt(quad(swing,swing,cr_gten))
	    do i=1,3
	      swingc(i)=swing(i)/rnn
	    enddo
	  else
	    do i=1,3
	      swingc(i)=wavec(i)
	    enddo
	  endif
	endif
c
	call trans(swingc,cr_fmat,swingp,3)
c
	if(wv_md_s.eq.MD_S_ALL .or.
     &	   wv_md_s.eq.MD_S_IND      ) then
	  if    (wv_func(1:1).eq.'b') then
	    call do_wave_md(wavep,swingp,disp,disc,box)
	  elseif(wv_func(1:1).eq.'s') then
	    call do_wave_md(wavep,swingp,disp,disc,sinus)
	  elseif(wv_func(1:1).eq.'t') then
	    call do_wave_md(wavep,swingp,disp,disc,triang)
	  endif
	elseif( wv_md_s.eq.MD_S_EVE .or.
     &	        wv_md_s.eq.MD_S_NON     ) then
	  if    (wv_func(1:1).eq.'b') then
	    call do_wave_all(wavep,swingp,disp,disc,box)
	  elseif(wv_func(1:1).eq.'s') then
	    call do_wave_all(wavep,swingp,disp,disc,sinus)
	  elseif(wv_func(1:1).eq.'t') then
	    call do_wave_all(wavep,swingp,disp,disc,triang)
	  endif
	endif
C
	end

c*****7*********************************************************
	subroutine do_wave_all(wavep,swingp,disp,disc,wave_func)
c-
c	Displaces all atoms by the corresponding
c	wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'random.inc'
	include       'microsta.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	integer        long,itrans,dens
	PARAMETER     (LONG=1,ITRANS=2,DENS=3)
c
	integer        i,j,icharge
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  phase = ran1(idum) * 360.0
	endif
c
c	open(55,file='fort.55',status='unknown')
	DO I=1,CR_NATOMS
	  if(w_latom(cr_iscat(i))) then
	    if( wv_md_s      .eq.MD_S_EVE       .or.
     &	        wv_md_s      .eq.MD_S_NON .and.
     &	        cr_md_next(i).ge.0                    ) then
	    DO J=1,3
	      UC(J)=CR_POS(J,I)
	    enddo
	    call trans(uc,cr_fmat,up,3)
	    arg=up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	    arg = arg + phase*rlam/360.
	    arg=amod(arg,rlam)/rlam
	    DIS=wave_func(AMP,ARG,amp0)
	    if(iwave.eq.DENS) then
	      if(ran1(idum).gt.dis) then
	        cr_iscat(i) = w_repl(cr_iscat(i))
	        if(cr_iscat(i).eq.0) then
	          cr_ndel = cr_ndel + 1
	        endif
	      endif
	    else
	      IF(.NOT. LACOUST) THEN
	        READ(CR_AT_LIS(CR_ISCAT(I)),1000) ICHARGE
	        IF(ICHARGE .LT. 0) DIS=-DIS
	      ENDIF
	      do j=1,3
	        disp(j)=dis*swingp(j)
	      enddo
	      call trans(disp,cr_gmat,disc,3)
c	write(55,333) (cr_pos(j,i),j=1,3)
c333	format(12(1x,f6.3))
	      DO J=1,3
	        CR_POS(J,I)=CR_POS(J,I)+DISc(j)
	        cr_dim(j,1)=amin1(cr_dim(j,1),cr_pos(j,i))
	        cr_dim(j,2)=amax1(cr_dim(j,2),cr_pos(j,i))
	      enddo
	    endif
	  endif
	  endif
	enddo
c	close(55)
1000	format(2x,i2)
	end
c*****7*********************************************************
	subroutine do_wave_md(wavep,swingp,disp,disc,wave_func)
c-
c	Displaces all atoms inside a microdomain by the corresponding
c	wave function.
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'microdis.inc'
	include       'microsta.inc'
	include       'random.inc'
	include       'waves.inc'
	include       'wink.inc'
c
	integer        long,itrans,dens
	PARAMETER     (LONG=1,ITRANS=2,DENS=3)
c
	integer        i,j,icharge
	real           uc(3),up(3),disp(3),disc(3)
	real           wavep(3),swingp(3),arg,dis
	real           phase_mc(0:MC_MAX_MICRO)
c
	real           ran1
c
	real           wave_func
	external       wave_func
c
	if(wv_phase_typ.eq.WV_RAND) then
	  if(wv_md_s.eq.MD_S_ALL) then
	    do i=1,mc_num
	      phase_mc(i) = ran1(idum) * 360.0
	    enddo
	  elseif(wv_md_s.eq.MD_S_IND) then
	    phase_mc(wv_md_nr) = ran1(idum) * 360.0
	  endif
	endif
	      
c
c------	Select atom if:
c	  type has been selected and
c	       all microdomains have been selected               or
c	       a specific microdomain type has been selected
c
	do i=1,cr_natoms
	  if(w_latom(cr_iscat(i)) .and. 
     &	     ( wv_md_s.eq.MD_S_ALL .and. cr_md_next(i).lt.0        .or.
     &	       wv_md_s.eq.MD_S_IND .and. cr_md_next(i).lt.0 .and.
     &         mc_type(-cr_md_next(i)).eq.wv_md_nr            ) ) then
	    if(wv_phase_typ.eq.WV_RAND) then
	      phase = phase_mc(-cr_md_next(i))
	    endif
	    do j=1,3
	      uc(j)=cr_pos(J,I)-mc_orig(j,-cr_md_next(i))
	    enddo
	    call trans(uc,cr_fmat,up,3)
	    arg=up(1)*wavep(1)+up(2)*wavep(2)+up(3)*wavep(3)
	    arg = arg + phase*rlam/360.
	    arg=amod(arg,rlam)/rlam
	    dis=wave_func(amp,arg,amp0)
	    if(iwave.eq.DENS) then
	      if(ran1(idum).gt.dis) then
	        cr_iscat(i) = w_repl(cr_iscat(i))
	        if(cr_iscat(i).eq.0) then
	          cr_ndel = cr_ndel + 1
	        endif
	      endif
	    else
	      if(.not. lacoust) then
	        read(cr_at_lis(cr_iscat(i)),1000) icharge
	        if(icharge .lt. 0) dis=-dis
	      endif
	      do j=1,3
	        disp(j)=dis*swingp(j)
	      enddo
	      call trans(disp,cr_gmat,disc,3)
	      do j=1,3
	        cr_pos(j,i)=cr_pos(j,i)+disc(J)
	        cr_dim(j,1)=amin1(cr_dim(j,1),cr_pos(j,i))
	        cr_dim(j,2)=amax1(cr_dim(j,2),cr_pos(j,i))
	      enddo
	    endif
	  endif
	enddo
1000	format(2x,i2)
	end
c*****7*********************************************************
	real function sinus (amp,arg,amp0)
c-
c	Sinusoidal wave function
c+
	implicit      none
c
	include      'wink.inc'
c
	real          amp,arg,amp0
c
	arg   = arg*zpi
 	sinus = amp*sin(arg) + amp0
	end
c*****7*********************************************************
	real function box (amp,arg,amp0)
c-
c	box shaped wave function
c+
	implicit      none
c
	include      'wink.inc'
c
	real          amp,arg,amp0
c
c------	For positive argument 0.25 is excluded, 0.75 included
c
	if(arg.ge.0.0 ) then
	  if(     arg  .lt. 0.25 .or. 0.75 .le. abs(arg) ) then
	    box = amp0 + amp
	  else
	    box = amp0
	  endif
c
c------	For negative argument the exclusions are reversed to ensure
c	continuity at arg=0.0
c
	else
	  if(    -arg  .le. 0.25 .or. 0.75 .lt. abs(arg) ) then
	    box = amp0 + amp
	  else
	    box = amp0
	  endif
	endif
	end
c*****7*********************************************************
	real function triang (AMP,ARG,amp0)
c-
c	triangular shaped wave function
c+
	implicit      none
c
	include      'wink.inc'
c
	real          amp,arg,amp0
c
	if(arg .ge. 0) then
	  triang = amp0 + amp * arg
	else
	  triang = amp0 + amp * (1. + arg)
	endif
	end
c*****7*********************************************************
	subroutine get_iscat(ianz,cpara,lpara,werte,maxw,lnew)
c-
c	Determines the scattering type of the parameter
c+
	implicit      none
c
	include       'config.inc'
	include       'crystal.inc'
	include       'crystal.cmm'
	include       'charact.inc'
	include       'errlist.inc'
c
	integer        maxw
c
	character*200  cpara(maxw),zeile
	integer        lpara(maxw)
	integer        i,j,l,ianz,jj,jp
	logical        lnew
	real           werte(maxw)
c
	real           berechne
c
c------	----Select which atoms are included in the wave
c
	ier_num = 0
	ier_typ = ER_NONE
	do i=1,maxw
	  werte(i) = 0.0
	enddo
	jj = 1
	jp = 0
	j  = 1
	do while( j.le.ianz .and. ier_num.eq.0 )
	  i  = ichar(cpara(j)(1:1))
	  if(cpara(j).eq.'all') then
	    werte(1) = -1
	    return
	  elseif(((a .le.i .and. i.le.z ).or. 
     &	          (aa.le.i .and. i.le.zz)   ) .and.
     &	          (index(cpara(j),'[').eq.0 )      )  then
	    call do_cap(cpara(j))
	    ier_num = -27
	    ier_typ = ER_APPL
	    do i=0,cr_nscat
	      if(cpara(j).eq.cr_at_lis(i)) then
	        werte(jj) = i
	        jj        = jj + 1
	        jp        = jp + 1
	        ier_num   = 0
	        ier_typ   = ER_NONE
	      endif
	    enddo
	    if(lnew) then
	      if(j.gt.1) then
	        if(cr_nscat.lt.MAXSCAT) then
	          cr_nscat             = cr_nscat + 1
	          cr_at_lis(cr_nscat)  = cpara(j)
	          cr_dw(cr_nscat)      = 0.0
	          werte(jj)            = cr_nscat
	          ier_num              = 0
	          ier_typ              = ER_NONE
	        else
	          ier_num              = -26
	          ier_typ              = ER_NONE
	        endif
	      endif
	    endif
	  else
	    zeile          = ' '
	    l              = lpara(j)
	    zeile(1  :1)   = '('
	    zeile(2  :l+1) = cpara(j)
	    zeile(l+2:l+2) = ')'
	    l              = l + 2
	    werte(jj)      = berechne(zeile,l)
	    if(ier_num.eq.0) then
	      if(0.le.nint(werte(jj)) .and. 
     &	         nint(werte(jj)).le.cr_nscat) then
	        jj      = jj + 1
	        ier_num = 0
	        ier_typ = ER_NONE
	      else
	        ier_num = -27
	        ier_typ = ER_APPL
	      endif
	    endif
	  endif
	  j = j + 1
	enddo
	ianz = max(ianz,jp)
c
	end
