/*---------------------------------------------------------------------------
 * X11 Interface for KUPLOT 3.0
 * Based on X11 interface of GNUPLOT 
 *
 * Version : 1.0
 * Date    : 15 Oct 96
 * Author  : Th. Proffen (proffen@rsc.anu.edu.au)
 * --------------------------------------------------------------------------*/

#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xresource.h>
 
#include <stdio.h>
#include <math.h>

#include "cx11.h"

/*---------------------------------------------------------------------------
 * Initialise X11 screen 
 *---------------------------------------------------------------------------*/

x11init_(ix,iy,ic,r,g,b)
unsigned int *r[Bcolors], *g[Bcolors], *b[Bcolors];
int *ix,*iy,*ic;
{
   XColor used, exact;
   int n;

   /* Open window or resize */

   if (!dpy) {
       W = *ix; H = *iy; preset (); }
   else {
       if (*ix != W  && *iy != H) {
          W = *ix; H = *iy; XResizeWindow(dpy, win, W, H); }}
          
   /* if necessary set new colormap */

   if (*ic > 0) {
     printf (" ------ > Creating individual X colormap ...\n");

     if (bmap) XFreeColormap(dpy, bmap);
     bmap=XCreateColormap(dpy,win,vis,AllocNone);

     /* Colours for lines, frame etc .. */

     for (n=0; n<Ncolors; n++) {
       if(XAllocNamedColor(dpy, bmap, color_values[n], &used, &exact))
         colors[n] = used.pixel;
       else {
         fprintf(stderr, " kuplot: Can't allocate %s\n", color_values[n]); }
     }
     
     /* bitmap colour */

     for (n=0; n<Bcolors; n++) {
        used.red   = (int)((int) *(r+n) / (float) (Bcolors) * MAX_INTENSITY);
        used.green = (int)((int) *(g+n) / (float) (Bcolors) * MAX_INTENSITY);
        used.blue  = (int)((int) *(b+n) / (float) (Bcolors) * MAX_INTENSITY);
        if (XAllocColor(dpy,bmap,&used) == 0) {
          fprintf (stderr, " kuplot: Unable to allocate all colours \n"); }
        cfeld[n]=(unsigned char) used.pixel; }

     /* set colormap */

     XSetWindowColormap(dpy, win, bmap);
     XInstallColormap(dpy, bmap);
     *ic=-(*ic);
   }

   /* set scaling factor between internal driver & window geometry */

   xscale = (double)W / 4096.;  yscale = (double)H / 4096.;

   /* create new pixmap & GC */

   if (gc) { XFreeGC(dpy, gc); XFreePixmap(dpy, pixmap); }
   pixmap = XCreatePixmap(dpy, root, W, H, D);
   gc = XCreateGC(dpy, pixmap, 0, (XGCValues *)0);
   XSetFont(dpy, gc, font->fid);

   /* set pixmap background */

   XSetForeground(dpy, gc, colors[0]);
   XFillRectangle(dpy, pixmap, gc, 0, 0, W, H);
   XSetBackground(dpy, gc, colors[0]);

   /* set new pixmap as window background */

   XSetWindowBackgroundPixmap(dpy, win, pixmap);

}

/*---------------------------------------------------------------------------
 * Exit X11
 *---------------------------------------------------------------------------*/

x11exit_()
{
   if (dpy) XCloseDisplay(dpy);
}

/*---------------------------------------------------------------------------
 * Getting font information like height and width
 *---------------------------------------------------------------------------*/

x11getfont_(chx,chy) 
float *chx,*chy;
{
   *chx = (font->max_bounds.rbearing - font->min_bounds.lbearing)/xscale;
   *chy = (font->max_bounds.ascent   + font->max_bounds.descent )/yscale;
}

/*---------------------------------------------------------------------------
 * Setting of KUPLOT font and font size
 *---------------------------------------------------------------------------*/

x11setfont_(fsize,fid)
int *fsize,*fid;
{
   char *font_name[80];

   switch (*fid) {
     case  1:sprintf (font_name,
            "-*-times-medium-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  2:sprintf (font_name,
            "-*-clean-medium-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  3:sprintf (font_name,
            "-*-symbol-medium-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  4:sprintf (font_name,
            "-*-helvetica-medium-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  5:sprintf (font_name,
            "-*-times-bold-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  6:sprintf (font_name,
            "-*-clean-bold-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  7:sprintf (font_name,
            "-*-symbol-bold-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  8:sprintf (font_name,
            "-*-helvetica-bold-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case  9:sprintf (font_name,
            "-*-times-medium-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 10:sprintf (font_name,
            "-*-clean-medium-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 11:sprintf (font_name,
            "-*-symbol-medium-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 12:sprintf (font_name,
            "-*-helvetica-medium-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 13:sprintf (font_name,
            "-*-times-bold-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 14:sprintf (font_name,
            "-*-clean-bold-r-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 15:sprintf (font_name,
            "-*-symbol-bold-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
     case 16:sprintf (font_name,
            "-*-helvetica-bold-o-*-*-*-%d-*-*-*-*-*-*",*fsize);break;
   }

   font = XLoadQueryFont(dpy, font_name);
   if (!font) {
      fprintf(stderr, " kuplot: can't load font '%s'\n", font_name);
      fprintf(stderr, " kuplot: using font '%s' instead.\n", FallbackFont);
      font = XLoadQueryFont(dpy, FallbackFont);
      if (!font) {
         fprintf(stderr, " kuplot: can't load font '%s'\n", FallbackFont);
         fprintf(stderr, " kuplot: no useable font - X11 aborted.\n");
         exit(1);
         }
      }
   vchar = font->ascent + font->descent;
   XSetFont(dpy, gc, font->fid);
}

/*---------------------------------------------------------------------------
 * This routine is called, after KUPLOT finished the graph
 *---------------------------------------------------------------------------*/

x11close_()
{
      	XClearWindow(dpy,win);
	XFlush (dpy);
}

/*---------------------------------------------------------------------------
 * This routine draws line from x1,y1 to x2,y2
 *---------------------------------------------------------------------------*/

x11line_ (x1,y1,x2,y2)
float *x1,*y1,*x2,*y2; 
{
	XDrawLine (dpy,pixmap,gc,X(*x1),Y(*y1),X(*x2),Y(*y2));
}

/*---------------------------------------------------------------------------
 * Draws circle 
 *---------------------------------------------------------------------------*/

x11circ_ (x1,y1,x2,y2)
float *x1,*y1,*x2,*y2; 
{
	XDrawArc (dpy,pixmap,gc,X(*x1),Y(*y1),X(*x2),Y(4095+*y2),0,23040);
}

/*---------------------------------------------------------------------------
 * Draws filled circle
 *--------------------------------------------------------------------------*/

x11fillcirc_ (x1,y1,x2,y2)
float *x1,*y1,*x2,*y2; 
{
	XDrawArc (dpy,pixmap,gc,X(*x1),Y(*y1),X(*x2),Y(4095+*y2),0,23040);
	XFillArc (dpy,pixmap,gc,X(*x1),Y(*y1),X(*x2),Y(4095+*y2),0,23040);
}

/*---------------------------------------------------------------------------
 * Draws filled area defined by given polygon
 *---------------------------------------------------------------------------*/

x11fillrect_ (x1,y1,x2,y2,x3,y3,x4,y4)
float *x1,*y1,*x2,*y2,*x3,*y3,*x4,*y4;
{
	XPoint points[5];
	int i;

	points[0].x = (short) X(*x1);
	points[0].y = (short) Y(*y1);
	points[1].x = (short) X(*x2);
	points[1].y = (short) Y(*y2);
	points[2].x = (short) X(*x3);
	points[2].y = (short) Y(*y3);
	points[3].x = (short) X(*x4);
	points[3].y = (short) Y(*y4);
	points[4].x = (short) X(*x1);
	points[4].y = (short) Y(*y1);

	XFillPolygon (dpy,pixmap,gc,&points,5,Convex,CoordModeOrigin);
}

/*---------------------------------------------------------------------------
 * Draws rectangle
 *---------------------------------------------------------------------------*/

x11rect_ (x1,y1,x2,y2,x3,y3,x4,y4)
float *x1,*y1,*x2,*y2,*x3,*y3,*x4,*y4;
{
	XPoint points[5];
	int i;

	points[0].x = (short) X(*x1);
	points[0].y = (short) Y(*y1);
	points[1].x = (short) X(*x2);
	points[1].y = (short) Y(*y2);
	points[2].x = (short) X(*x3);
	points[2].y = (short) Y(*y3);
	points[3].x = (short) X(*x4);
	points[3].y = (short) Y(*y4);
	points[4].x = (short) X(*x1);
	points[4].y = (short) Y(*y1);

	XDrawLines (dpy,pixmap,gc,&points,5,CoordModeOrigin);
}

/*---------------------------------------------------------------------------
 * Draws text at position x,y
 *---------------------------------------------------------------------------*/

x11text_ (chfeld,ilen,x,y,jmode)
float *x,*y;
unsigned int *chfeld[132];
int *ilen,*jmode;
{
	char  str[132];
	int   sh,sw,sww,i;

   	for (i=0 ; i < *ilen ; i++)
	{
   		str[i] = (char) *(chfeld+i);
   	}
	str[i]='\0';

	if (*jmode > 0) {
        	sw = XTextWidth(font, str, *ilen);
	        sww= sw;
        	switch(*jmode) {
           	case 1:   sw = 0;     break;
           	case 2:   sw = -sw/2; break;
           	case 3:   sw = -sw;   break;
           	}
   		XDrawString (dpy,pixmap, gc, X(*x)+sw, Y(*y), str, *ilen);
		*x = (float) (sww  /xscale);
		*y = (float) (vchar/yscale);
	}
	else {
		sh=(*ilen-1)*vchar;
        	switch(*jmode) {
           	case -1:   sh = 0;     break;
           	case -2:   sh = -sh/2; break;
           	case -3:   sh = -sh;   break;
           	}
		for (i=0 ; i < *ilen ; i++)
		{
			sw=XTextWidth(font, &str[i], 1);
			sw=-sw/2;
			XDrawString (dpy,pixmap,gc,X(*x)+sw,Y(*y)+i*vchar+sh,
			             &str[i],1);
		}
	}

}

/*---------------------------------------------------------------------------
 * Sets foreground colour (rgb)
 *---------------------------------------------------------------------------*/

x11colrgb_ (r,g,b)
unsigned int *r,*g,*b;
{
	XColor used;

        used.red   = (int)( *r / (float) (Bcolors) * MAX_INTENSITY);
        used.green = (int)( *g / (float) (Bcolors) * MAX_INTENSITY);
        used.blue  = (int)( *b / (float) (Bcolors) * MAX_INTENSITY);

        if (bmap) {
          if (XAllocColor(dpy,bmap,&used) == 0) {
            fprintf (stderr, " kuplot: Unable to allocate all colours \n"); 
	    return; }}
        else {
          if (XAllocColor(dpy,cmap,&used) == 0) {
            fprintf (stderr, " kuplot: Unable to allocate all colours \n"); 
	    return; }}

	XSetForeground (dpy, gc, used.pixel);
}

/*---------------------------------------------------------------------------
 * Sets foreground colour
 *---------------------------------------------------------------------------*/

x11col_ (icol)
unsigned int *icol; 
{
	XSetForeground (dpy, gc, colors[*icol]);
}

/*---------------------------------------------------------------------------
 * Sets line type
 *---------------------------------------------------------------------------*/

x11ltyp_ (ityp,lwid)
unsigned int *ityp; 
float *lwid;
{
	int lt, width, type;

        lt = (*ityp%8)+2;
        width = (int) (*lwid*2);
        if (lt == 3)
           type = LineSolid;
        else {
           type = LineOnOffDash;
           XSetDashes(dpy, gc, 0, dashes[lt], strlen(dashes[lt]));
           }
        XSetLineAttributes( dpy,gc, width, type, CapButt, JoinBevel);
}

/*---------------------------------------------------------------------------
 * Get mouse coordinates
 *---------------------------------------------------------------------------*/

x11in_(wx,wy,qret)
int *wx,*wy,*qret;
{
	Window rwin, cwin;
	int rx,ry,ret;

	do
	{
		XQueryPointer(dpy,win,&rwin,&cwin,&rx,&ry,wx,wy,qret);  
	}
	while (*qret == 0);   

        do
        {
                XQueryPointer(dpy,win,&rwin,&cwin,&rx,&ry,wx,wy,&ret);
        }
        while (ret != 0);

	*wx = *wx / xscale;
	*wy = 4095 - *wy / yscale;
}

/*---------------------------------------------------------------------------
 * Draw the bitmap
 *---------------------------------------------------------------------------*/
x11bitmap_(z,nx,ny,ix,iy,jx,jy,zzmin,zzmax,shear,r,g,b)
unsigned char *z;
unsigned int *r,*g,*b;
float *zzmin, *zzmax, *shear;
int  *nx,*ny,*ix,*iy,*jx,*jy;
{
	XImage		*img;
	XColor 		used;
	unsigned char	cfarbe,*img_feld, *image_feld;
	float		img_skal_x, img_skal_y, shear_neu;
	int		img_x, img_y, ixx, iyy, nxx, nyy;
	int		image_x, image_y, pos_x, pos_y, lx, ly, xoff;

	if (D != 8)
	{
		fprintf (stderr,
		"Display with colour depth of %d bit not supported !\n",D);
		return;
	}

        used.red   = (int)( *r / (float) (Bcolors) * MAX_INTENSITY);
        used.green = (int)( *g / (float) (Bcolors) * MAX_INTENSITY);
        used.blue  = (int)( *b / (float) (Bcolors) * MAX_INTENSITY);
        if (XAllocColor(dpy,bmap,&used) == 0) {
          fprintf (stderr, " kuplot: Unable to allocate all colours \n"); 
	  return; }

	img_x    = X(*jx)-X(*ix); 
	img_y    = Y(4095+*jy)-Y(4095+*iy);
	img_feld = (unsigned char *)malloc (img_x*img_y*sizeof(char));
	if (!img_feld)
	{
		fprintf (stderr," kuplot: No Memory for Bitmap \n");
		return;
	}
	img_skal_x=(float) (img_x) / (float)(*nx); 
	img_skal_y=(float) (img_y) / (float)(*ny); 

	for (iyy=0 ; iyy < img_y ; iyy++)
	{
		for (ixx=0 ; ixx < img_x ; ixx++)
		{
			nxx=(int) (ixx / img_skal_x);
			nyy=(int) (iyy / img_skal_y);
			cfarbe= *(z+sizeof(char)*(nyy*(*nx)+nxx));
			*(img_feld+sizeof(char)*((iyy*img_x)+ixx))=
			  cfeld[(int)cfarbe];
		}
	}

	if ( *shear != 0.0 )
	{
		shear_neu = (float) H / (float) W * (24.70/17.45) * (*shear);
		image_x = img_x + abs ((int) ((float) img_y / shear_neu));
		image_y = img_y;
		image_feld = (unsigned char *)malloc 
		             (image_x*image_y*sizeof(char));
        	if (!image_feld)
        	{
                	fprintf (stderr," kuplot: No Memory for shearing \n");
                	return;
        	}
		memset (image_feld,used.pixel,image_x*image_y*sizeof(char));
		xoff = 0;
		if ( *shear < 0) 
		{
			xoff = abs ((int) ((float) img_y / shear_neu));
		}
		for (iyy=0 ; iyy < img_y ; iyy++)
		{
			for (ixx=0 ; ixx < img_x ; ixx++)
			{
				lx = ixx + xoff +
				     (int) ((float) (img_y-iyy) / shear_neu);
				ly = iyy;
				*(image_feld+sizeof(char)*((ly*image_x)+lx))=
				*(img_feld+sizeof(char)*((iyy*img_x)+ixx));
			}
		}
		img=XCreateImage(dpy,vis,D,ZPixmap,0,image_feld,image_x,
		                 image_y,8,0);
		pos_x   = X(*ix) - xoff;
		pos_y   = Y(*iy) - img_y;
        	XPutImage(dpy,pixmap,gc,img,0,0,pos_x,pos_y,image_x,image_y);
        	XDestroyImage(img);   
		free(img_feld);
	}
	else
	{
		img=XCreateImage(dpy,vis,D,ZPixmap,0,img_feld,img_x,img_y,8,0);
		pos_x   = X(*ix); pos_y   = Y(*iy)-img_y;
		image_x = img_x;  image_y = img_y;

        	XPutImage(dpy,pixmap,gc,img,0,0,pos_x,pos_y,image_x,image_y);
        	XDestroyImage(img);   
	}
}

/*-----------------------------------------------------------------------------
 *   preset - determine options, open display, create window
 *---------------------------------------------------------------------------*/

char *getenv();

preset() 
{
   char *display = getenv("DISPLAY"),  *home = getenv("HOME");

   /*---open display---------------------------------------------------------*/

   dpy = XOpenDisplay(display); 
   if (!dpy) {
      fprintf(stderr, " kuplot: unable to open display '%s'\n", display);
      fprintf(stderr, " kuplot: X11 aborted.\n");
      exit(1);
      }

   scr  = DefaultScreen(dpy);
   vis  = DefaultVisual(dpy,scr);
   D    = DefaultDepth(dpy,scr);
   root = DefaultRootWindow(dpy);

   /*---do some presets and open window--------------------------------------*/

   pr_font();
   pr_color();
   pr_window();

} 

/*-----------------------------------------------------------------------------
 *   pr_color - determine color values
 *---------------------------------------------------------------------------*/

pr_color() 
{
   unsigned long black = BlackPixel(dpy, scr);
   unsigned long white = WhitePixel(dpy,scr);
   XColor used, exact;
   char *v; 
   int n;

   if (vis->class == StaticGray) Mono++;

   if (!Mono) 
   {
      cmap = DefaultColormap(dpy, scr);
      for (n=0; n<Ncolors; n++) {
	 v = color_values[n];
	 if (XAllocNamedColor(dpy, cmap, v, &used, &exact))
	    colors[n] = used.pixel;
	 else {
	    fprintf(stderr, " kuplot: can't allocate %s\n", v);
	    fprintf(stderr, " kuplot: reverting to monochrome\n");
	    Mono++; break;
	    }
	 }
      }
   if (Mono) {
      colors[0] = (Rv) ? black : white ;
      for (n=1; n<Ncolors; n++)  colors[n] = (Rv) ? white : black;
      }
}

/*-----------------------------------------------------------------------------
 *   pr_font - determine font          
 *---------------------------------------------------------------------------*/

pr_font() 
{
    font = XLoadQueryFont(dpy, FallbackFont);
    if (!font) {
       fprintf(stderr, " kuplot: can't load font '%s'\n", FallbackFont);
       fprintf(stderr, " kuplot: no useable font - X11 aborted.\n");
       exit(1);
    }
    vchar = font->ascent + font->descent;
}

/*-----------------------------------------------------------------------------
 *   pr_window - create window 
 *---------------------------------------------------------------------------*/

pr_window() 
{
   XSizeHints hints;

   win = XCreateSimpleWindow(dpy, root, gX, gY, W, H, 2, colors[1], colors[0]);

   hints.flags = PPosition;
   hints.x = gX; hints.y = gY;
   XSetNormalHints(dpy, win, &hints);

   XStoreName(dpy, win, Class);

   XSelectInput(dpy, win, StructureNotifyMask);
   XMapWindow(dpy, win);
   
}
/*---------------------------------------------------------------------------
 * Intensity percentages to RGB
 *---------------------------------------------------------------------------*/

RGB PctToRGB(rr, gg, bb)
float   rr, gg, bb;
{
        RGB     rgb;

        if (rr > 1.0)
                rr = 1.0;
        if (gg > 1.0)
                gg = 1.0;
        if (bb > 1.0)
                bb = 1.0;

        rgb.r = (int)(0.5 + rr * MAX_INTENSITY);
        rgb.g = (int)(0.5 + gg * MAX_INTENSITY);
        rgb.b = (int)(0.5 + bb * MAX_INTENSITY);
        return rgb;
}

