c*****7*****************************************************************
c
c	These routines set the frame parameters for KUPLOT. Each frame
c	can be seen as individual plotting area.
c
c	Version : 1.0
c	Date    : 16 Oct 96
c
c	Author  : Th. Proffen  (proffen@rsc.anu.edu.au)
c
c*****7*****************************************************************
	subroutine set_afra (zeile,lp)
c+
c	Sets active frame for input parameters
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=2)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifr
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  write (*,1000) iframe
	elseif (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  if (ier_num.ne.0) return
	  ifr=nint(werte(1))
	  if (ifr.lt.1 .or. ifr.gt.iaf) then
	    ier_num = -15
	    ier_typ = ER_APPL
	  else
	    iframe = ifr
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
1000	format (' ------ > Active frame : ',i3)
	end
c*****7*****************************************************************
	subroutine set_bfra (zeile,lp)
c+
c	Set backgound color for frames
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=5)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifr
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  call show_frames
	elseif (ianz.eq.4) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  if (ier_num.ne.0) return
	  ifr=nint(werte(1))
	  if (ifr.lt.1 .or. ifr.gt.iaf) then
	    ier_num = -15
	    ier_typ = ER_APPL
	  else
	    if (werte(2).le.1.0 .and. werte(2).ge.0.0  .and.
     &          werte(3).le.1.0 .and. werte(3).ge.0.0  .and.
     &          werte(4).le.1.0 .and. werte(4).ge.0.0       ) then
	      frback(ifr,1) = werte(2)
	      frback(ifr,2) = werte(3)
	      frback(ifr,3) = werte(4)
	    else
	      ier_num = -18
	      ier_typ = ER_APPL
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine set_cfra (zeile,lp)
c+
c	Copy plot parameters of one frame to other frame(s)
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=maxframe+2)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifrom,ito,i
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.ge.2) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  if (ier_num.ne.0) return
	  ifrom=nint(werte(1))
	  do i=2,ianz
	    ito=nint(werte(i))
	    if (ifrom.lt.1 .or. ifrom.gt.iaf .or.
     &	        ito  .lt.1 .or. ito  .gt.iaf .or.
     &	        ifrom.eq.ito                     ) then
	      ier_num = -15
	      ier_typ = ER_APPL
	      return
	    else	
	      call copy_frame(ifrom,ito)
	    endif
	  enddo
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine set_sfra (zeile,lp)
c+
c	Sets frame sizes and locations
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=6)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifr
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  call show_frames
	elseif (ianz.eq.5) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  if (ier_num.ne.0) return
c
	  ifr=nint(werte(1))
	  if (ifr.lt.1 .or. ifr.gt.iaf) then
	    ier_num = -15
	    ier_typ = ER_APPL
	  else
	    if (werte(2).lt.werte(4) .and. werte(3).lt.werte(5)) then
	      frame(ifr,1) = werte(2)
	      frame(ifr,2) = werte(3)
	      frame(ifr,3) = werte(4)
	      frame(ifr,4) = werte(5)
	    else
	      ier_num = -7
	      ier_typ = ER_APPL
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine set_kfra (zeile,lp)
c+
c	Set datasets to be displayed in each frame
c	or give textfile for text frames.
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=maxkurvtot+2)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifr,ikk,i
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  call show_frames
	elseif (ianz.ge.2) then
	  call ber_params(1,cpara,lpara,werte,maxw)	
	  ifr=nint(werte(1))
	  if (ier_num.ne.0) return
	  if (ifr.lt.1 .or. ifr.gt.iaf) then
	    ier_num = -15
	    ier_typ = ER_APPL
	  else
	    call del_params(1,ianz,cpara,lpara,maxw)
	    call ber_params(ianz,cpara,lpara,werte,maxw)
	    if (ier_num.eq.0) then
	      do i=1,maxkurv
	        infra(ifr,i)=0
	      enddo
	      do i=1,ianz
	        ikk=nint(werte(i))
	        if (ikk.lt.1 .or. ikk.gt.(iz-1)) then
	          ier_num = -4
	          ier_typ = ER_APPL
	        else
	          infra(ifr,i)=ikk
	        endif
	      enddo
	    else
	      call no_error
	      call do_build_name(ianz,cpara,lpara,werte,maxw,1)
	      if (ier_num.ne.0) return
	      infra(ifr,1)=-1
	      ftext(ifr)=cpara(1)
	    endif
	  endif
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
	end
c*****7*****************************************************************
	subroutine set_nfra (zeile,lp)
c+
c	Set mumber of frames to be used
c-
	implicit	none
c
	include		'kuplot.inc'
	include		'errlist.inc'
c
	integer		maxw
	parameter	(maxw=2)
c
	character*(*)	zeile
	integer		lp
c
	character*200	cpara(maxw)
	integer		lpara(maxw)
	integer		ianz,ifr,idummy
	real		werte(maxw)
c
	call get_params(zeile,ianz,cpara,lpara,maxw,lp)
	if (ier_num.ne.0) return
c
	if (ianz.eq.0) then
	  call show_frames
	elseif (ianz.eq.1) then
	  call ber_params(ianz,cpara,lpara,werte,maxw)	
	  idummy=nint(werte(1))
	  if (ier_num.ne.0) return
c
	  if (idummy.le.0 .or. idummy.gt.maxframe) then
	    ier_num = -17
	    ier_typ = ER_APPL
	  else 
	    iaf = idummy
	    do ifr=1,iaf
	      plmass(ifr)=-9999.0
	      call copy_frame (1,ifr)
	    enddo
c
c-----------default frames
c
	    if     (iaf.eq.1) then
	      frame (1,1) = 0.0
	      frame (1,2) = 0.0
	      frame (1,3) = 1.0
	      frame (1,4) = 1.0
	    elseif (iaf.eq.2) then
	      frame (1,1) = 0.0
	      frame (1,2) = 0.0
	      frame (1,3) = 0.5
	      frame (1,4) = 1.0
c
	      frame (2,1) = 0.5
	      frame (2,2) = 0.0
	      frame (2,3) = 1.0
	      frame (2,4) = 1.0
	    elseif (iaf.eq.3) then
	      frame (1,1) = 0.0
	      frame (1,2) = 0.0
	      frame (1,3) = 0.333
	      frame (1,4) = 1.0
c
	      frame (2,1) = 0.333
	      frame (2,2) = 0.0
	      frame (2,3) = 0.666
	      frame (2,4) = 1.0
c
	      frame (3,1) = 0.666
	      frame (3,2) = 0.0
	      frame (3,3) = 1.0
	      frame (3,4) = 1.0
	    elseif (iaf.eq.4) then
	      frame (1,1) = 0.0
	      frame (1,2) = 0.0
	      frame (1,3) = 0.5
	      frame (1,4) = 0.5
c
	      frame (2,1) = 0.5
	      frame (2,2) = 0.0
	      frame (2,3) = 1.0
	      frame (2,4) = 0.5
c
	      frame (3,1) = 0.0
	      frame (3,2) = 0.5
	      frame (3,3) = 0.5
	      frame (3,4) = 1.0
c
	      frame (4,1) = 0.5
	      frame (4,2) = 0.5
	      frame (4,3) = 1.0
	      frame (4,4) = 1.0
	    elseif (iaf.eq.5 .or. iaf.eq.6) then
	      frame (1,1) = 0.0
	      frame (1,2) = 0.0
	      frame (1,3) = 0.333
	      frame (1,4) = 0.5
c
	      frame (2,1) = 0.333
	      frame (2,2) = 0.0
	      frame (2,3) = 0.666
	      frame (2,4) = 0.5
c
	      frame (3,1) = 0.666
	      frame (3,2) = 0.0
	      frame (3,3) = 1.0
	      frame (3,4) = 0.5
c
	      frame (4,1) = 0.0
	      frame (4,2) = 0.5
	      frame (4,3) = 0.333
	      frame (4,4) = 1.0
c
	      frame (5,1) = 0.333
	      frame (5,2) = 0.5
	      frame (5,3) = 0.666
	      frame (5,4) = 1.0
c
	      frame (6,1) = 0.666
	      frame (6,2) = 0.5
	      frame (6,3) = 1.0
	      frame (6,4) = 1.0
	    else
	      write (*,2000) iaf
	    endif
	  endif    
	else
	  ier_num = -6
	  ier_typ = ER_COMM
	endif
c
2000	format (' ****WARN****No default for ',i2,' frames, use sfra ..')  
	end
c*****7*****************************************************************
	subroutine copy_frame (if1,if2)
c+
c	Copy parameters associated with frame if1 to frame if2
c-
	implicit	none
c
	include		'kuplot.inc'
c
	integer		if1,if2
	integer		j,ihl
c
	achse(if2,1) = achse(if1,1)
	achse(if2,2) = achse(if1,2)
	titel(if2,1) = titel(if1,1)
	titel(if2,2) = titel(if1,2)
	ftext(if2)   = ftext(if1)
	plmass(if2)  = plmass(if1)
	yskal(if2)   = yskal(if1)
	ex(if2,1)    = ex(if1,1)
	ex(if2,2)    = ex(if1,2)
	ey(if2,1)    = ey(if1,1)
	ey(if2,2)    = ey(if1,2)
	t (if2,1)    = t (if1,1)
	t (if2,2)    = t (if1,2)
	t0(if2,1)    = t0(if1,1)
	t0(if2,2)    = t0(if1,2)
	shear(if2)   = shear(if1)
	sfl(if2)     = sfl(if1)
	igrid(if2)   = igrid(if1)
	ibox(if2)    = ibox(if1)
	ifname(if2)  = ifname(if1)
	frjust(if2)  = frjust(if1)
	frback(if2,1)= frback(if1,1)
	frback(if2,2)= frback(if1,2)
	frback(if2,3)= frback(if1,3)
c
	do j=1,10
	  fonsize(if2,j) = fonsize(if1,j)
	  fon_id (if2,j) = fon_id (if1,j)
	  foncol (if2,j) = foncol (if1,j)
	enddo
c
	do j=1,maxan
	  antext(if2,j) = antext(if1,j)
	  anx(if2,j)    = anx(if1,j)
	  any(if2,j)    = any(if1,j)
	  anjust(if2,j) = anjust(if1,j)
	enddo
c
	do ihl=1,maxhl
	  nz    (if2,ihl) = nz    (if1,ihl)
	  z_min (if2,ihl) = z_min (if1,ihl)
	  z_inc (if2,ihl) = z_inc (if1,ihl)
	  iho   (if2)     = iho   (if1)
	enddo
c
	end
